'''
Copyright 2012 Dmytro Karpenko

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
'''
import datetime
import GeneratorGlobals as GGlobals

#===================================================

def gen_gwa_output(output_filename):

    if output_filename:
        filename_to_use = "%s_input_files.mdl" % output_filename
    else:
        filename_to_use = "input_files.mdl"
    # Flushing input file list to a file
    FILE = open(filename_to_use,"w")
    for i in GGlobals.global_files_list:
        FILE.write("%s | %s\n" % (str(i["file_id"]), str(i["file_size"])))
    FILE.close()

    if output_filename:
        filename_to_use = "%s_output_files.mdl" % output_filename
    else:
        filename_to_use = "output_files.mdl"
    # Flushing output file list to a file
    FILE = open(filename_to_use,"w")
    for i in range(0,sum(GGlobals.num_of_out_files_values)):
        FILE.write("%s | %s\n" % (str(i), str(GGlobals.size_of_out_files_values[i])))
    FILE.close()

    if output_filename:
        filename_to_use = "%s.gwf" % output_filename
    else:
        filename_to_use = "jobs.gwf"
    # Flushing jobs list to a file
    FILE = open(filename_to_use,"w")

    inter_sum = 0
    outindex = 0

    for i in range(0,len(GGlobals.jobs_file_sets)):
        # "-" -- for ignored fields
        
        # The fields are:
        
        # JobID, starts from 1
        # SubmitTime -- relative to the first job
        # WaitTime -- ignored
        # RunTime -- our walltime, converted to seconds
        # NProc -- our jobs use only one processor element
        # Average CPU time used -- ignored
        # Used memory -- ignored
        # ReqNProcs -- requested number of processor elements, always 1
        # Requested time -- ignored
        # Requested memory -- ignored
        # Status -- ignored
        # UserID -- ignored
        # GroupID -- ignored
        # ExecutableID -- ignored
        # QueueID -- ignored
        # PartitionID -- ignored
        # OrigSiteID -- ignored
        # LastRunSiteID -- ignored
        # JobStructure -- always UNITARY
        # JobStructureParams -- ignored
        # UsedNetwork -- ignored
        # UsedLocalDiskSpace -- ignored
        # UsedResources -- ignored
        # ReqPlatform -- OS is only Linux, architecture and OSVersion can be different
        # ReqNetwork -- ignored
        # ReqLocalDiskSpace -- ignored
        # RequestedResources -- ignored
        # VOID -- always "ATLAS"
        # ProjectID -- ignored
        inter_sum += GGlobals.intertimes_values[i]
        line = "%s | %s | - | %s | 1 | - | - | 1 | - | - | - | - | - | - | - | - | - | - | UNITARY |" % \
                (str(i+1), str(inter_sum), str(GGlobals.walltimes_values[i] * 60))
        line += " - | - | - | - | Linux | - | - | - | ATLAS | - | "
        
        # Now, the IDs of input files
        for sym in sorted(GGlobals.jobs_file_sets[i]):
            line += str(sym) + " "
        line += "| "

        # and output files, if the job has any
        for k in range(0,GGlobals.num_of_out_files_values[i]):
            line += str(outindex) + " "
            outindex += 1
        line += "|\n"
        
        FILE.write(line)
    
    FILE.close()

#===================================================

def gen_swf_output(output_filename, extended = False):
    
    # Flushing input file list to a file
    if extended:
        if output_filename:
            filename_to_use = "%s_input_files.swf.txt" % output_filename
        else:
            filename_to_use = "input_files.swf.txt"
        FILE = open(filename_to_use, "w")
        for i in GGlobals.global_files_list:
            line = str(i["file_id"]) + " " + str(i["file_size"]) + "\n"
            FILE.write(line)
        FILE.close()

    # Flushing output file list to a file
    if extended:
        if output_filename:
            filename_to_use = "%s_output_files.swf.txt" % output_filename
        else:
            filename_to_use = "output_files.swf.txt"
        FILE = open(filename_to_use, "w")
        for i in range(0,sum(GGlobals.num_of_out_files_values)):
            line = str(i) + " " + str(GGlobals.size_of_out_files_values[i]) + "\n"
            FILE.write(line)
        FILE.close()

    # Flushing jobs list to a file
    if output_filename:
        filename_to_use = "%s.swf.txt" % output_filename
    else:
        filename_to_use = "ndgf_atlas_model.swf.txt"
    FILE = open(filename_to_use,"w")

    inter_sum = 0
    outindex = 0
    for i in range(0,len(GGlobals.jobs_file_sets)):
        # "-1" -- for ignored fields
        
        # The fields are:
        
        # JobID, starts from 1
        # SubmitTime -- relative to the first job
        # Ignored
        # RunTime -- our walltime converted to seconds
        # NProc -- our jobs use only one processor element
        # 2 ignored fields
        # Required number of processors -- also always 1
        # From 9 to 18 fields -- ignored

        inter_sum += GGlobals.intertimes_values[i]
        line = "%s %s -1 %s 1 -1 -1 1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1" \
                % (str(i+1), str(inter_sum), str(GGlobals.walltimes_values[i] * 60), )
        
        if extended:
            line += " "
            # Now, the IDs of input files
            s_list = sorted(GGlobals.jobs_file_sets[i])
            for sym in sorted(s_list[:-1]):
                line += str(sym) + ","
            line += str(s_list[-1])
            line += " "

            # and output files, if the job has any
            for k in range(0,GGlobals.num_of_out_files_values[i] - 1):
                line += str(outindex) + ","
                outindex += 1
            line += str(outindex)
            outindex += 1
        line += "\n"
        FILE.write(line)
    
    FILE.close()

#===================================================

def gen_dax_output(output_filename):
    if output_filename:
        filename_to_use = "%s.xml" % output_filename
    else:
        filename_to_use = "ndgf_atlas_dax.xml"
    FILE = open(filename_to_use,"w")
    # Print the header info
    print>>FILE, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    print>>FILE, "<!-- generated: %s -->" % datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%S')
    print>>FILE, "<!-- by NDGF-ATLAS workload generator -->"
    # Open the job container element
    print>>FILE, "<adag xmlns=\"http://pegasus.isi.edu/schema/DAX\""
    print>>FILE, "  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
    print>>FILE, "  xsi:schemaLocation=\"http://pegasus.isi.edu/schema/DAX"
    print>>FILE, "                      http://pegasus.isi.edu/schema/dax-2.1.xsd\""
    print>>FILE, "  version=\"2.1\" count=\"1\" index=\"0\" name=\"NDGF-ATLAS\" jobCount=\"%d\" fileCount=\"%d\" childCount=\"0\">" \
            % (len(GGlobals.jobs_file_sets), sum(GGlobals.num_of_out_files_values) + len(GGlobals.global_files_list))
            
    outindex = 0
    # Loop through jobs
    for i in range(0,len(GGlobals.jobs_file_sets)):
        # Don't forget walltime value should be in seconds!
        # We generate it in minutes by default
        print>>FILE, "  <job id=\"%d\" namespace=\"SG\" name=\"job%d\" version=\"1.0\" runtime=\"%d\">" \
                % ( i + 1, i + 1, GGlobals.walltimes_values[i] * 60)
        # Print all the input files the job requires
        # Don't forget size should be in bytes!
        # We generate it in MB by default
        for sym in sorted(GGlobals.jobs_file_sets[i]):
            print>>FILE, "    <uses file=\"input%d\" link=\"input\" register=\"true\" transfer=\"true\" optional=\"false\" type=\"data\" size=\"%d\"/>" \
                    % (sym, GGlobals.global_files_list[sym]["file_size"] * 1024 * 1024)

        # Print all the output files the job produces
        # Don't forget size should be in bytes!
        # We generate it in MB by default
        for k in range(0,GGlobals.num_of_out_files_values[i]):
            print>>FILE, "    <uses file=\"output%d\" link=\"output\" register=\"true\" transfer=\"true\" optional=\"false\" type=\"data\" size=\"%d\"/>" \
                    % (outindex, GGlobals.size_of_out_files_values[outindex] * 1024 * 1024)
            outindex += 1
            
        print>>FILE, "  </job>"
        
    # Close the job container element
    print>>FILE, "</adag>"
    FILE.close()
